/****************************************************************
 *
 *  Copyright (c) 1995, Case Western Reserve University
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms are permitted
 *  provided that the above copyright notice and this paragraph are
 *  duplicated in all such forms and that any documentation,
 *  advertising materials, and other materials related to such
 *  distribution and use acknowledge that the software was developed
 *  by the Case Western Reserve University.  The name of the Case
 *  Western Reserve University may not be used to endorse or promote
 *  products derived from this software without specific prior written
 *  permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 *  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 *  WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  This program is intended not to be used for commercial purposes
 *  without permission of the Case Western Reserve University 
 *  and without the permission of those who developed the YACC grammer.
 *
 *  AUTHOR:
 *      Michael Knieser
 *      Case Western Reserve University
 *      Department of Computer and Electrical Engineering
 *      10900 Euclid Av.
 *      Cleveland, OH 44118
 *      USA
 *      Phone: (216) 932-7135
 *      e-mail: knieser@alpha.ces.cwru.edu
 *  Version : 0.1
 *  Date : 8/15/95
 *
 ****************************************************************/

#include	<stdio.h>
#include	<ctype.h>
#include	<string.h>

#define	YYSTYPE	char*

#include	"y.tab.h"

#define	DEBUG	0
#define	printd	if ( DEBUG == 1 ) printf

#define YY_INPUT(buf,result,max_size) \
                { \
                int c = tolower(getc( yyin )); \
                result = c == EOF ? 0 : 1; \
                buf[0] = (char) c; \
                } \

int		line_number = 1;
YYSTYPE		Convert_To_Lower ();
void		Inc_Scope ();
void		Dec_Scope ();
YYSTYPE		Make_Scope_String ();
YYSTYPE		Make_String ();
YYSTYPE		Combine ();
YYSTYPE		Return_Newlines ();
YYSTYPE		Return_ScopeText ();
void		to_uppercase();
void		to_lowercase();
void		comment();
void		count();
int		q();
int		p();

long		size_of_temp_string;
int		keyword_case;
int		identifier_case;
YYSTYPE		comment_string;
YYSTYPE		temp_lex;
int		newline;
int		no_comments;

#include	"lex.yy.c"
#include	"y.tab.c"


/**************************************************
Return_ScopeText :
	temp = scope_text;
 **************************************************/
YYSTYPE Return_ScopeText ( next_string )
YYSTYPE	*next_string;
{
	YYSTYPE	temp;
	YYSTYPE	temp_next;
	int	end_comment = -1;
	int	temp_int = 0;

	if ( no_comments == 0 )
	{
		temp_next = Make_String(*next_string);

		while ( temp_next[temp_int] != '\0' )
		{
			if ( temp_next[temp_int] == '-' && temp_next[temp_int+1] == '-' )
			{
				while ( temp_next[temp_int] != '\0' && temp_next[temp_int] != '\n' )
					temp_int++;
				if ( temp_next[temp_int] == '\n' )
					end_comment = temp_int;
			}
			else if ( temp_next[temp_int] != '\n' )
				break;

			if (temp_next[temp_int] != '\0')
				temp_int++;
		}

		if ( end_comment != -1 )
		{
			*next_string = Make_String(&temp_next[end_comment+1]);
			temp_next[end_comment+1] = '\0';
			temp = Combine(temp_next,scope_text);
		}
		else
			temp = scope_text;
		free ( temp_next );
	}
	else
		temp = scope_text;

	return ( temp );
}


/**************************************************
Return_Newlines :
 **************************************************/
YYSTYPE Return_Newlines ( number_of )
int	number_of;
{
	YYSTYPE	temp;
	int	temp_int;

	temp = NULL;

	temp = (char *)calloc((number_of+1),sizeof(char));
	if ( temp == NULL )
	{
		fprintf(stderr,"MEMORY ERROR!!!!!\n");
		exit(-1);
	}
	else
	{
		for ( temp_int=0; temp_int < number_of; temp_int++)
			temp[temp_int] = '\n';
		temp[temp_int] = '\0';
	}

	return ( temp );
}


/**************************************************
Convert_To_Lower : Converts a passed string to all lower case.
 **************************************************/
YYSTYPE Convert_To_Lower ( string1 )
YYSTYPE		string1;
{
	YYSTYPE	temp;
	int	length = 0;
	int	loop = 0;

	length = strlen(string1)+1;

	temp = (char *)calloc(length,sizeof(char));
	if ( temp == NULL )
		printf("MEMORY ERROR!!!!!\n");
	else
	{
		while ( string1[loop] != '\0' )
		{
			if ( isupper(string1[loop]) == 1 )
				temp[loop] = tolower(string1[loop]);
			else
				temp[loop] = string1[loop];
			loop++;
		}
			temp[loop] = '\0';
	}

	return ( temp );
}


/**************************************************
Inc_Scope : Increments the scope variable.
 **************************************************/
void Inc_Scope ()
{
	scope++;
	if ( scope_text != NULL )
		free ( scope_text );
	scope_text = Make_Scope_String(scope);
}


/**************************************************
Dec_Scope : Decrements the scope variable.
 **************************************************/
void Dec_Scope ()
{
	scope--;
	if ( scope_text != NULL )
		free ( scope_text );
	scope_text = Make_Scope_String(scope);
}


/**************************************************
Make_Scope_String : Returns a tabed string with 'number' tabs.
 **************************************************/
YYSTYPE Make_Scope_String ( number )
int	number;
{
	YYSTYPE	temp;
	int	length = 0;
	int	loop;

	if ( number <= 0 )
	{
		printd("ERROR in passed value (%d)(%s)...\n",number,yytext);
		number = 0;
	}

	length = number + 1;

	temp = (char *)calloc(length,sizeof(char));

	if ( temp == NULL )
		printf("MEMORY ERROR!!!!!\n");
	else
	{
		for (loop=0; loop<number; loop++)
		{
			temp[loop] = '\t';
		}
		temp[loop] = '\0';
	}

	return ( temp );
}


/**************************************************
Make_String : Returns a memory allocated string pointer.
 **************************************************/
YYSTYPE Make_String ( string1 )
YYSTYPE		string1;
{
	YYSTYPE	temp;
	int	length = 0;

	length = strlen(string1)+1;

	temp = (char *)calloc(length,sizeof(char));
	if ( temp == NULL )
		printf("MEMORY ERROR!!!!!\n");
	else
		strcpy(temp,string1);

	if ( length > (size_of_temp_string/6) )
	{
		free(temp_string);
		size_of_temp_string *= 6;
		temp_string = (char *)
			calloc(size_of_temp_string,sizeof(char));

		if ( temp_string == NULL )
			printf("\n\nERROR: Cannot allocate memory for 'temp_string'...");
	}

	return ( temp );
}


/**************************************************
Combine : Returns a combined memory allocated string of the two passed strings.
 **************************************************/
YYSTYPE Combine ( string1, string2 )
YYSTYPE		string1;
YYSTYPE		string2;
{
	YYSTYPE	temp;
	int	length = 0;

	if ( string1 != NULL )
		length += strlen(string1) + 1;
	if ( string2 != NULL )
		length += strlen(string2) + 1;

	temp = (char *)calloc(length,sizeof(char));

	if ( temp == NULL )
		printf("MEMORY ERROR!!!!!\n");
	else if ( string1 != NULL && string2 != NULL )
	{
		strcpy(temp,string1);
		strcat(temp,string2);
	}
	else if ( string1 == NULL && string2 != NULL )
		strcpy(temp,string2);
	else if ( string1 != NULL && string2 == NULL )
		strcpy(temp,string1);
	else
		temp = NULL;

	if ( length > (size_of_temp_string/6) )
	{
		free(temp_string);
		size_of_temp_string *= 6;
		temp_string = (char *)
			calloc(size_of_temp_string,sizeof(char));

		if ( temp_string == NULL )
			printf("\n\nERROR: Cannot allocate memory for 'temp_string'...");
	}

	return ( temp );
}


/**************************************************
yywrap : YACC subroutine to indicate a REAL EOF.
int yywrap()
{
	return(-1);
}
 **************************************************/


/**************************************************
to_uppercase : converts 'yytext' to ALL upper case characters.
 **************************************************/
void to_uppercase()
{
	int n;

	for (n = 0; yytext[n] != '\0'; n++)
		yytext[n] = toupper(yytext[n]);
}


/**************************************************
to_lowercase : converts 'yytext' to ALL lower case characters.
 **************************************************/
void to_lowercase()
{
	int n;

	for (n = 0; yytext[n] != '\0'; n++)
		yytext[n] = tolower(yytext[n]);
}


/**************************************************
q : passes 'yytext' to 'yylval' for KEYWORDS.
		free ( comment_string );
 **************************************************/
int q(token)
int token;
{
	if ( keyword_case == 1 )
		to_uppercase();
		
	if ( strcmp(comment_string,"") == 0 || no_comments == 1 )
		yylval = Make_String(yytext);
	else
	{
		yylval = Combine(comment_string,yytext);
		free(comment_string);
		comment_string = Make_String("");
	}
	return (token);
}


/**************************************************
p : passes 'yytext' to 'yylval' which is on the YACC stack.
		free ( comment_string );
 **************************************************/
int p(token)
int token;
{
	if ( token == Identifier )
	{
		if ( identifier_case == 1 )
			to_uppercase();
	}

	if ( strcmp(comment_string,"") == 0 || no_comments == 1 )
		yylval = Make_String(yytext);
	else
	{
		yylval = Combine(comment_string,yytext);
		free(comment_string);
		comment_string = Make_String("");
	}
	return (token);
}


/**************************************************
 **************************************************/
int main(argc,argv)
int argc;
char *argv[];
{
	int loop;

	keyword_case = 0;
	identifier_case = 0;
	no_comments = 0;

	for (loop=1; loop<argc; loop++)
		fprintf(stderr, "(%d): %s\n",loop,argv[loop]);
	for (loop=1; loop<argc; loop++)
	{
		if ( strcmp (argv[loop], "-uk") == 0 )
			keyword_case = 1;
		else if ( strcmp (argv[loop], "-lk") == 0 )
			keyword_case = 0;
		else if ( strcmp (argv[loop], "-ui") == 0 )
			identifier_case = 1;
		else if ( strcmp (argv[loop], "-li") == 0 )
			identifier_case = 0;
		else if ( strcmp (argv[loop], "-NC") == 0 )
			no_comments = 1;
		else if ( strcmp (argv[loop], "-h") == 0 ||
			  strcmp (argv[loop], "--help") == 0 )
		{
			printf("\nvhdl-nice [-NC] [-uk] [-lk] [-ui] [-li] [< vhdl_file] [> out_file]");
			printf("\n------------------------------------------------------------------");
			printf("\n\t          -NC		: NO Commets!");
			printf("\n\t          -uk		: Upper case key words");
			printf("\n\t          -lk		: Lower case key words(default)");
			printf("\n\t          -ui		: Upper case identifier");
			printf("\n\t          -li		: Lower case identifier(default)");
			printf("\n\n");
			exit (0);
		}
	}

	size_of_temp_string = 1234;
	temp_string = (char *)calloc(1234,sizeof(char));

	if ( temp_string == NULL )
		printf("\n\nERROR: Cannot allocate memory for 'temp_string'...");

	scope = 0;
	scope_text = Make_Scope_String(scope);
	comment_string = Make_String("");
	newline = 0;

	yyparse();

	free ( temp_string );

	return (0);
}
